@@ -1,6 +1,8 @@
 #include "cache.h"
+
 #include "config.h"
 #include "log.h"
+#include "util.h"
 
 #include <sys/stat.h>
 
@@ -44,14 +46,14 @@ static char *CacheSystem_calc_dir(const char *url)
         xdg_cache_home = path_append(home, xdg_cache_home_default);
     }
     if (mkdir
-        (xdg_cache_home, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
-        && (errno != EEXIST)) {
+            (xdg_cache_home, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+            && (errno != EEXIST)) {
         lprintf(fatal, "mkdir(): %s\n", strerror(errno));
     }
     char *cache_dir_root = path_append(xdg_cache_home, "/httpdirfs/");
     if (mkdir
-        (cache_dir_root, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
-        && (errno != EEXIST)) {
+            (cache_dir_root, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+            && (errno != EEXIST)) {
         lprintf(fatal, "mkdir(): %s\n", strerror(errno));
     }
 
@@ -75,7 +77,7 @@ static char *CacheSystem_calc_dir(const char *url)
     char *escaped_url = curl_easy_escape(c, url, 0);
     char *full_path = path_append(cache_dir_root, escaped_url);
     if (mkdir(full_path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
-        && (errno != EEXIST)) {
+            && (errno != EEXIST)) {
         lprintf(fatal, "mkdir(): %s\n", strerror(errno));
     }
     FREE(fn);
@@ -87,6 +89,8 @@ static char *CacheSystem_calc_dir(const char *url)
 
 void CacheSystem_init(const char *path, int url_supplied)
 {
+    lprintf(cache_lock_debug,
+            "thread %x: initialise cf_lock;\n", pthread_self());
     if (pthread_mutex_init(&cf_lock, NULL)) {
         lprintf(fatal, "cf_lock initialisation failed!\n");
     }
@@ -103,12 +107,12 @@ void CacheSystem_init(const char *path, int url_supplied)
      * Check if directories exist, if not, create them
      */
     if (mkdir(META_DIR, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
-        && (errno != EEXIST)) {
+            && (errno != EEXIST)) {
         lprintf(fatal, "mkdir(): %s\n", strerror(errno));
     }
 
     if (mkdir(DATA_DIR, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
-        && (errno != EEXIST)) {
+            && (errno != EEXIST)) {
         lprintf(fatal, "mkdir(): %s\n", strerror(errno));
     }
 
@@ -119,8 +123,8 @@ void CacheSystem_init(const char *path, int url_supplied)
          */
         sonic_path = path_append(META_DIR, "rest/");
         if (mkdir
-            (sonic_path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
-            && (errno != EEXIST)) {
+                (sonic_path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+                && (errno != EEXIST)) {
             lprintf(fatal, "mkdir(): %s\n", strerror(errno));
         }
         FREE(sonic_path);
@@ -130,8 +134,8 @@ void CacheSystem_init(const char *path, int url_supplied)
          */
         sonic_path = path_append(DATA_DIR, "rest/");
         if (mkdir
-            (sonic_path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
-            && (errno != EEXIST)) {
+                (sonic_path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+                && (errno != EEXIST)) {
             lprintf(fatal, "mkdir(): %s\n", strerror(errno));
         }
         FREE(sonic_path);
@@ -144,7 +148,7 @@ void CacheSystem_init(const char *path, int url_supplied)
  * \brief read a metadata file
  * \return 0 on success, errno on error.
  */
-static int Meta_read(Cache * cf)
+static int Meta_read(Cache *cf)
 {
     FILE *fp = cf->mfp;
     rewind(fp);
@@ -232,7 +236,7 @@ file!\n");
  *  - -1 on error,
  *  - 0 on success
  */
-static int Meta_write(Cache * cf)
+static int Meta_write(Cache *cf)
 {
     FILE *fp = cf->mfp;
     rewind(fp);
@@ -275,7 +279,7 @@ static int Meta_write(Cache * cf)
  * \details We use sparse creation here
  * \return exit on failure
  */
-static void Data_create(Cache * cf)
+static void Data_create(Cache *cf)
 {
     int fd;
     int mode;
@@ -322,7 +326,7 @@ static long Data_size(const char *fn)
  *  - negative values on error,
  *  - otherwise, the number of bytes read.
  */
-static long Data_read(Cache * cf, uint8_t * buf, off_t len, off_t offset)
+static long Data_read(Cache *cf, uint8_t *buf, off_t len, off_t offset)
 {
     if (len == 0) {
         lprintf(error, "requested to read 0 byte!\n");
@@ -375,7 +379,7 @@ static long Data_read(Cache * cf, uint8_t * buf, off_t len, off_t offset)
         }
     }
 
-  end:
+end:
 
     lprintf(cache_lock_debug,
             "thread %x: unlocking seek_lock;\n", pthread_self());
@@ -393,7 +397,7 @@ static long Data_read(Cache * cf, uint8_t * buf, off_t len, off_t offset)
  *  - -1 when the data file does not exist
  *  - otherwise, the number of bytes written.
  */
-static long Data_write(Cache * cf, const uint8_t * buf, off_t len,
+static long Data_write(Cache *cf, const uint8_t *buf, off_t len,
                        off_t offset)
 {
     if (len == 0) {
@@ -433,7 +437,7 @@ static long Data_write(Cache * cf, const uint8_t * buf, off_t len,
         lprintf(error, "fwrite(): encountered error!\n");
     }
 
-  end:
+end:
     lprintf(cache_lock_debug,
             "thread %x: unlocking seek_lock;\n", pthread_self());
     PTHREAD_MUTEX_UNLOCK(&cf->seek_lock);
@@ -495,7 +499,7 @@ static Cache *Cache_alloc()
 /**
  * \brief free a cache data structure
  */
-static void Cache_free(Cache * cf)
+static void Cache_free(Cache *cf)
 {
     if (pthread_mutex_destroy(&cf->seek_lock)) {
         lprintf(fatal, "could not destroy seek_lock!\n");
@@ -575,7 +579,7 @@ void Cache_delete(const char *fn)
 {
     if (CONFIG.mode == SONIC) {
         Link *link = path_to_Link(fn);
-        fn = link->sonic_id;
+        fn = link->sonic.id;
     }
 
     char *metafn = path_append(META_DIR, fn);
@@ -601,18 +605,19 @@ void Cache_delete(const char *fn)
  *  -   0 on success
  *  -   -1 on failure, with appropriate errno set.
  */
-static int Data_open(Cache * cf)
+static int Data_open(Cache *cf)
 {
     char *datafn = path_append(DATA_DIR, cf->path);
     cf->dfp = fopen(datafn, "r+");
-    FREE(datafn);
     if (!cf->dfp) {
         /*
          * Failed to open the data file
          */
         lprintf(error, "fopen(%s): %s\n", datafn, strerror(errno));
+        FREE(datafn);
         return -1;
     }
+    FREE(datafn);
     return 0;
 }
 
@@ -622,7 +627,7 @@ static int Data_open(Cache * cf)
  *  -   0 on success
  *  -   -1 on failure, with appropriate errno set.
  */
-static int Meta_open(Cache * cf)
+static int Meta_open(Cache *cf)
 {
     char *metafn = path_append(META_DIR, cf->path);
     cf->mfp = fopen(metafn, "r+");
@@ -642,7 +647,7 @@ static int Meta_open(Cache * cf)
  * \brief Create a metafile
  * \return exit on error
  */
-static void Meta_create(Cache * cf)
+static void Meta_create(Cache *cf)
 {
     char *metafn = path_append(META_DIR, cf->path);
     cf->mfp = fopen(metafn, "w");
@@ -652,6 +657,11 @@ static void Meta_create(Cache * cf)
          */
         lprintf(fatal, "fopen(%s): %s\n", metafn, strerror(errno));
     }
+    if (fclose(cf->mfp)) {
+        lprintf(error,
+                "cannot close metadata after creation: %s.\n",
+                strerror(errno));
+    }
     FREE(metafn);
 }
 
@@ -667,7 +677,7 @@ int Cache_create(const char *path)
     } else if (CONFIG.mode == SINGLE) {
         fn = curl_easy_unescape(NULL, this_link->linkname, 0, NULL);
     } else if (CONFIG.mode == SONIC) {
-        fn = this_link->sonic_id;
+        fn = this_link->sonic.id;
     } else {
         lprintf(fatal, "Invalid CONFIG.mode\n");
     }
@@ -683,12 +693,6 @@ int Cache_create(const char *path)
 
     Meta_create(cf);
 
-    if (fclose(cf->mfp)) {
-        lprintf(error,
-                "cannot close metadata after creation: %s.\n",
-                strerror(errno));
-    }
-
     if (Meta_open(cf)) {
         Cache_free(cf);
         lprintf(error, "cannot open metadata file, %s.\n", fn);
@@ -734,8 +738,8 @@ Cache *Cache_open(const char *fn)
             "thread %x: locking cf_lock;\n", pthread_self());
     PTHREAD_MUTEX_LOCK(&cf_lock);
 
-    if (link->cache_opened) {
-        link->cache_opened++;
+    if (link->cache_ptr) {
+        link->cache_ptr->cache_opened++;
 
         lprintf(cache_lock_debug,
                 "thread %x: unlocking cf_lock;\n", pthread_self());
@@ -755,7 +759,7 @@ Cache *Cache_open(const char *fn)
             return NULL;
         }
     } else if (CONFIG.mode == SONIC) {
-        if (Cache_exist(link->sonic_id)) {
+        if (Cache_exist(link->sonic.id)) {
 
             lprintf(cache_lock_debug,
                     "thread %x: unlocking cf_lock;\n", pthread_self());
@@ -781,7 +785,7 @@ Cache *Cache_open(const char *fn)
      * Set the path for the local cache file, if we are in sonic mode
      */
     if (CONFIG.mode == SONIC) {
-        fn = link->sonic_id;
+        fn = link->sonic.id;
     }
 
     cf->path = strndup(fn, MAX_PATH_LEN);
@@ -821,7 +825,8 @@ Cache *Cache_open(const char *fn)
      */
     if (cf->content_length > Data_size(fn)) {
         lprintf(error, "metadata inconsistency %s, \
-cf->content_length: %ld, Data_size(fn): %ld.\n", fn, cf->content_length, Data_size(fn));
+cf->content_length: %ld, Data_size(fn): %ld.\n", fn, cf->content_length,
+                Data_size(fn));
         Cache_free(cf);
 
         lprintf(cache_lock_debug,
@@ -853,7 +858,7 @@ cf->content_length: %ld, Data_size(fn): %ld.\n", fn, cf->content_length, Data_si
         return NULL;
     }
 
-    cf->link->cache_opened = 1;
+    cf->cache_opened = 1;
     /*
      * Yup, we just created a circular loop. ;)
      */
@@ -865,15 +870,15 @@ cf->content_length: %ld, Data_size(fn): %ld.\n", fn, cf->content_length, Data_si
     return cf;
 }
 
-void Cache_close(Cache * cf)
+void Cache_close(Cache *cf)
 {
     lprintf(cache_lock_debug,
             "thread %x: locking cf_lock;\n", pthread_self());
     PTHREAD_MUTEX_LOCK(&cf_lock);
 
-    cf->link->cache_opened--;
+    cf->cache_opened--;
 
-    if (cf->link->cache_opened > 0) {
+    if (cf->cache_opened > 0) {
 
         lprintf(cache_lock_debug,
                 "thread %x: unlocking cf_lock;\n", pthread_self());
@@ -893,6 +898,8 @@ void Cache_close(Cache * cf)
         lprintf(error, "cannot close data file %s.\n", strerror(errno));
     }
 
+    cf->link->cache_ptr = NULL;
+
     lprintf(cache_lock_debug,
             "thread %x: unlocking cf_lock;\n", pthread_self());
     PTHREAD_MUTEX_UNLOCK(&cf_lock);
@@ -903,7 +910,7 @@ void Cache_close(Cache * cf)
  * \brief Check if a segment exists.
  * \return 1 if the segment exists
  */
-static int Seg_exist(Cache * cf, off_t offset)
+static int Seg_exist(Cache *cf, off_t offset)
 {
     off_t byte = offset / cf->blksz;
     return cf->seg[byte];
@@ -916,7 +923,7 @@ static int Seg_exist(Cache * cf, off_t offset)
  * \param[in] i 1 for exist, 0 for doesn't exist
  * \note Call this after downloading a segment.
  */
-static void Seg_set(Cache * cf, off_t offset, int i)
+static void Seg_set(Cache *cf, off_t offset, int i)
 {
     off_t byte = offset / cf->blksz;
     cf->seg[byte] = i;
@@ -938,7 +945,7 @@ static void *Cache_bgdl(void *arg)
 
     uint8_t *recv_buf = CALLOC(cf->blksz, sizeof(uint8_t));
     lprintf(debug, "thread %x spawned.\n ", pthread_self());
-    long recv = path_download(cf->fs_path, (char *) recv_buf, cf->blksz,
+    long recv = Link_download(cf->link, (char *) recv_buf, cf->blksz,
                               cf->next_dl_offset);
     if (recv < 0) {
         lprintf(error, "thread %x received %ld bytes, \
@@ -946,8 +953,8 @@ which does't make sense\n", pthread_self(), recv);
     }
 
     if ((recv == cf->blksz) ||
-        (cf->next_dl_offset ==
-         (cf->content_length / cf->blksz * cf->blksz))) {
+            (cf->next_dl_offset ==
+             (cf->content_length / cf->blksz * cf->blksz))) {
         Data_write(cf, recv_buf, recv, cf->next_dl_offset);
         Seg_set(cf, cf->next_dl_offset, 1);
     } else {
@@ -965,12 +972,14 @@ error.\n", recv, cf->blksz);
             "thread %x: unlocking w_lock;\n", pthread_self());
     PTHREAD_MUTEX_UNLOCK(&cf->w_lock);
 
-    pthread_detach(pthread_self());
+    if (pthread_detach(pthread_self())) {
+        lprintf(error, "%s\n", strerror(errno));
+    };
     pthread_exit(NULL);
 }
 
 long
-Cache_read(Cache * cf, char *const output_buf, const off_t len,
+Cache_read(Cache *cf, char *const output_buf, const off_t len,
            const off_t offset_start)
 {
     long send;
@@ -1017,7 +1026,7 @@ Cache_read(Cache * cf, char *const output_buf, const off_t len,
 
     uint8_t *recv_buf = CALLOC(cf->blksz, sizeof(uint8_t));
     lprintf(debug, "thread %x: spawned.\n ", pthread_self());
-    long recv = path_download(cf->fs_path, (char *) recv_buf, cf->blksz,
+    long recv = Link_download(cf->link, (char *) recv_buf, cf->blksz,
                               dl_offset);
     if (recv < 0) {
         lprintf(error, "thread %x received %ld bytes, \
@@ -1030,7 +1039,7 @@ which does't make sense\n", pthread_self(), recv);
      * Condition 2: offset is the last segment
      */
     if ((recv == cf->blksz) ||
-        (dl_offset == (cf->content_length / cf->blksz * cf->blksz))) {
+            (dl_offset == (cf->content_length / cf->blksz * cf->blksz))) {
         Data_write(cf, recv_buf, recv, dl_offset);
         Seg_set(cf, dl_offset, 1);
     } else {
@@ -1047,12 +1056,11 @@ error.\n", recv, cf->blksz);
     /*
      * ----------- Download the next segment in background -----------------
      */
-  bgdl:
-    {
+bgdl: {
     }
     off_t next_dl_offset = round_div(offset_start, cf->blksz) * cf->blksz;
     if ((next_dl_offset > dl_offset) && !Seg_exist(cf, next_dl_offset)
-        && next_dl_offset < cf->content_length) {
+            && next_dl_offset < cf->content_length) {
         /*
          * Stop the spawning of multiple background pthreads
          */
