@@ -5,16 +5,20 @@
  * \file link.h
  * \brief link related structures and functions
  */
-#include "config.h"
-#include "util.h"
-#include <curl/curl.h>
 
-/** \brief Link type */
 typedef struct Link Link;
+typedef struct LinkTable LinkTable;
 
 #include "cache.h"
+#include "config.h"
+#include "network.h"
+#include "sonic.h"
 
-/** \brief the link type */
+#include <curl/curl.h>
+
+/**
+ * \brief the link type
+ */
 typedef enum {
     LINK_HEAD = 'H',
     LINK_DIR = 'D',
@@ -23,30 +27,14 @@ typedef enum {
     LINK_UNINITIALISED_FILE = 'U'
 } LinkType;
 
-/** \brief for storing downloaded data in memory */
-typedef struct {
-    char *data;
-    size_t size;
-} DataStruct;
-
-/** \brief specify the type of data transfer */
-typedef enum {
-    FILESTAT = 's',
-    DATA = 'd'
-} TransferType;
-
-/** \brief for storing the link being transferred, and metadata */
-typedef struct {
-    TransferType type;
-    int transferring;
-    Link *link;
-} TransferStruct;
-
 /**
  * \brief link table type
  * \details index 0 contains the Link for the base URL
  */
-typedef struct LinkTable LinkTable;
+struct LinkTable {
+    int num;
+    Link **links;
+};
 
 /**
  * \brief Link type data structure
@@ -64,31 +52,10 @@ struct Link {
     LinkTable *next_table;
     /** \brief CURLINFO_FILETIME obtained from the server */
     long time;
-    /** \brief How many times associated cache has been opened */
-    int cache_opened;
     /** \brief The pointer associated with the cache file */
     Cache *cache_ptr;
-    /**
-     * \brief Sonic id field
-     * \details This is used to store the followings:
-     *  - Arist ID
-     *  - Album ID
-     *  - Song ID
-     *  - Sub-directory ID (in the XML response, this is the ID on the "child"
-     *    element)
-     */
-    char *sonic_id;
-    /**
-     * \brief Sonic directory depth
-     * \details This is used exclusively in ID3 mode to store the depth of the
-     * current directory.
-     */
-    int sonic_depth;
-};
-
-struct LinkTable {
-    int num;
-    Link **links;
+    /** \brief Stores *sonic related data */
+    Sonic sonic;
 };
 
 /**
@@ -109,7 +76,7 @@ LinkTable *LinkSystem_init(const char *raw_url);
 /**
  * \brief Set the stats of a link, after curl multi handle finished querying
  */
-void Link_set_file_stat(Link * this_link, CURL * curl);
+void Link_set_file_stat(Link *this_link, CURL *curl);
 
 /**
  * \brief create a new LinkTable
@@ -117,12 +84,19 @@ void Link_set_file_stat(Link * this_link, CURL * curl);
 LinkTable *LinkTable_new(const char *url);
 
 /**
- * \brief download a link
+ * \brief download a path
  * \return the number of bytes downloaded
  */
 long path_download(const char *path, char *output_buf, size_t size,
                    off_t offset);
 
+/**
+ * \brief Download a Link
+ * \return the number of bytes downloaded
+ */
+long Link_download(Link *link, char *output_buf, size_t req_size,
+                   off_t offset);
+
 /**
  * \brief find the link associated with a path
  */
@@ -136,7 +110,7 @@ LinkTable *path_to_Link_LinkTable_new(const char *path);
 /**
  * \brief dump a link table to the disk.
  */
-int LinkTable_disk_save(LinkTable * linktbl, const char *dirn);
+int LinkTable_disk_save(LinkTable *linktbl, const char *dirn);
 
 /**
  * \brief load a link table from the disk.
@@ -145,9 +119,9 @@ LinkTable *LinkTable_disk_open(const char *dirn);
 
 /**
  * \brief Download a link's content to the memory
- * \warning You MUST free the memory field in DataStruct after use!
+ * \warning You MUST free the memory field in TransferStruct after use!
  */
-DataStruct Link_to_DataStruct(Link * head_link);
+TransferStruct Link_download_full(Link *head_link);
 
 /**
  * \brief Allocate a LinkTable
@@ -158,15 +132,15 @@ LinkTable *LinkTable_alloc(const char *url);
 /**
  * \brief free a LinkTable
  */
-void LinkTable_free(LinkTable * linktbl);
+void LinkTable_free(LinkTable *linktbl);
 
 /**
  * \brief print a LinkTable
  */
-void LinkTable_print(LinkTable * linktbl);
+void LinkTable_print(LinkTable *linktbl);
 
 /**
  * \brief add a Link to a LinkTable
  */
-void LinkTable_add(LinkTable * linktbl, Link * link);
+void LinkTable_add(LinkTable *linktbl, Link *link);
 #endif
